% solve Schrodinger equation for two-dimensional square potential well
% using finite difference
% Kyle Konrad 11/22/09
% update Dhrubo 11/25/09


% N = x-axis degrees of freedom
% M = N = y-axis degrees of freeom
% k = number of eigenvalues to be found by eigs
% opt = 0: don't find eigenvectors
%       1: find eigenvectors

function [v,e,errors] = twodeigs(N,k,sigma,opt,showPlot)

M = N;

x = (1:N)/(N+1); % interior x points
y = (1:M)/(M+1); % interior y points

hx = 1/(N+2); % horizontal lattice separation
hy = 1/(M+2); % verticle lattice separation

% [xs,ys] = meshgrid([0 x 1],[0 y 1]); % 2-D mesh (including boundary points)


% A is the finite difference approximation of the Laplacian in 2-D
% where columns in region are stacked into a single column of A

%   ^                  
% y |                  
%   |      1           
%   |  1  -4   1       
%   |      1                
%   |                  
%    ------------>
%              x

A = sparse((N+2)*(M+2),(N+2)*(M+2));
%top-left corner i.e. for point (x, y) = (0, 0)
A(1,1) = -4;  %self
A(1,1+1) = 1;   %east
A(1,1+(M+2)) = 1; %north

%bottom-right corner i.e. for point (x, y) = (1, 1)
A((N+2)*(M+2),(N+2)*(M+2)) = -4;  %self
A((N+2)*(M+2),(N+2)*(M+2)-1) = 1; %west
A((N+2)*(M+2),(N+1)*(M+2)) = 1;   %south

for n = 2:(N+2)*(M+2)-1
    A(n,n) = -4; %self
    if mod(n, M+2) ~= 1; A(n,n-1)= 1; end; %north
    if mod(n, M+2) ~= M+1; A(n,n+1)= 1; end; %south
    if n+(M+2)<=(N+2)*(M+2); A(n,n+(M+2))= 1; end %east
    if n-(M+2)>=1; A(n,n-(M+2))= 1; end %west
end

A = 1/(hx*hy)*A;
opts.disp = 0;

if opt
    [v,e] = eigs(-A,k,sigma,opts); % compute eigenfunctions and eigenvalues
    % sort eigenvalues and eigenvectors
    v((M+2)*(N+2)+1,:) = diag(e);
    t = sortrows(v',(M+2)*(N+2)+1)';
    e = t((M+2)*(N+2)+1,:)'; % column vector of sorted eigenvalues
    v = t(1:(M+2)*(N+2),:); % corresponding eigenvectors
    v = reshape(v,M+2,N+2,k); % solutions reshaped into 3d array
else e = eigs(-A,k,sigma,opts); % compute eigenvalues
     v = [];
end

% plotting code from Barnett
if showPlot
    figure; ny=ceil(sqrt(k));
    for i=1:k^2, subplot(ny, ny, i);
        imagesc(x, y, squeeze(S(:,:,i)));
        axis equal tight; caxis(max(abs(caxis))*[-1 1]);
        title(sprintf('E_{%d}=%.6g', i, e(i)));
    end
end

% eigenvalue analysis
[expx,expy] = meshgrid((1:ceil(sqrt(k))).^2);
exps = expx + expy;
exps = pi^2*sort(exps(1:k)');
errors = abs(e - exps)./exps;
% figure; plot(e, 'b--'); hold on; plot(exps, 'r'); title('eigenvalues');